/*
  This file is part of bcalc.
  bcalc is a multi-ui RPN calculator
  Copyright (C) 2007  Patrik Nilsson
  
  bcalc is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  bcalc is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with bcalc; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* this is the calculator part really, with some extras */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bcalc.h"
#include "wraper-ui.h"
#include "drmath.h"

Stack_node *stack_start;
Stack_node *stack_stop;

void dr_init( void )
{
    /* Note: the doctor needs to initalize some stuff */
    stack_start = ( Stack_node * )malloc( sizeof( Stack_node ) );
    stack_stop = ( Stack_node * )malloc( sizeof( Stack_node ) );
    stack_start->next = stack_stop;
    stack_stop->next = NULL;
}

void dr_parse_input( char *input, size_t input_size )
{
    /* Note: This function is used to determine what the input was,
       And what to do with it */
    int x, len;
    float new_num;
    char operand;

    /* remove eventual ',' and make them '.' */
    len = strlen( input );
    for( x = 0; x <= len; x++ )
        if( input[x] == ',' )
            input[x] = '.';
    
    /* real parser part */ 
    if( sscanf( input, "%f", &new_num ) && len > 0 )
    {
        add_element( new_num );
    }
    else if( sscanf( input, "%c", &operand ) && len == 1 )
    {
        switch( operand )
        {
        case '+': /* math ones */
            dr_do_math( '+' );
            break;
        case '-':
            dr_do_math( '-' );
            break;
        case '*':
            dr_do_math( '*' );
            break;
        case '/':
            dr_do_math( '/' );
            break;
        case 'd': /* delete top */
            del_element();
            break;
        case 's':
            print_stack();
            break;
        case 'p': /* print top value */
            if( stack_start->next->next != NULL )
                dr_print_result( stack_start->next->value );
            else
                ui_print( "stack is empty" );
            break;
        case 'q': /* exit bcalc */
            ui_print( "good bye" );
            running = 0;
            break;
        case 'c': /* clear stack */
            clear_stack();
            break;
        default:
            ui_print( "Error: unknown command" );
            break;
        }
    }
    else
    {
        ui_print( "Error: illegal command '%s'", input );
    }
    
#ifdef DEBUG
    ui_print( "'%s'", input );
    ui_print( "%d lenght", strlen( input ));
#endif
}

int test_double( double subj )
{
    /* a simple test to see if subj is a double(TRUE) or a int(FALSE) */
    double test_d;
    int test_i;

    test_i = subj;
    test_d = test_i;
    if( test_d != subj )
        return TRUE;
    else
        return FALSE;
}

void dr_do_math( char type )
{
    /* Note: this is really the calculator part */
    double value1, value2, result;
    int ok;

    if( stack_start->next->next != NULL &&
        stack_start->next->next->next != NULL )
    {
        value1 = stack_start->next->value;
        value2 = stack_start->next->next->value;

        switch( type )
        {
        case '+':
            result = value2 + value1;
            ok = TRUE;
            break;
        case '-':
            result = value2 - value1;
            ok = TRUE;
            break;
        case '*':
            result = value2 * value1;
            ok = TRUE;
            break;
        case '/':
            result = value2 / value1;
            ok = TRUE;
            break;
        default:
            ui_print( "warning: dr_do_math (drmath.c) got type '%c'", type );
            ok = FALSE;
            result = 0; /* to stop some nagging during compile */
            break;
        }
        
        /* clean up */
        if( ok )
        {
            del_element();
            del_element();
            add_element( result );

            dr_print_result( result );
        }
    }
    else
        ui_print( "stack is empty/not big enought" );
}

void dr_print_result( double value )
{
    /* Note: prints the result of a operand */
    int stripper;

    if( test_double( value ) )
        ui_print( "-> %f", value );
    else
    {
        stripper = value;
        ui_print( "-> %i", stripper );
    }
}

/* stack stuff */
void add_element( double value )
{
    /* Note: adds a new element to the stack */
    Stack_node *node;

    node = ( Stack_node * )malloc( sizeof( Stack_node ) );
    node->next = stack_start->next;
    node->value = value;
    stack_start->next = node;
#ifdef DEBUG
    ui_print( "deb: next->%i,value->%f", &node->next, node->value );
#endif
}

void del_element( void )
{
    /* Note: deletes the top element in the stack */
    Stack_node *node;
    
    if( stack_start->next->next == NULL )
        ui_print( "stack is empty" );
    else
    {
        node = stack_start->next;
        stack_start->next = node->next;
        free( node );
    }
}

void print_stack( void )
{
    /* Note: prints all elements in the stack */
    Stack_node *node;
    int x, stripper;
    
    x = 0;
    node = stack_start->next;
    while( node->next != NULL )
    {
        if( test_double( node->value ) )
            ui_print( "%i: %f", x, node->value );
        else
        {
            stripper = node->value;
            ui_print( "%i: %i", x, stripper );
        }
        x++;
        node = node->next;
    }

    if( stack_start->next->next == NULL )
        ui_print( "stack is empty" );
}

void clear_stack( void )
{
    /* Note: clears the stack */
    int done;
    
    done = FALSE;

    while( !done )
    {
        if( stack_start->next->next == NULL )
            done = TRUE;
        else
            del_element();
    }
    ui_print( "stack cleared" );
}
